@@ -1,5 +1,6 @@
 """Core data classes and functions."""
 
+from collections.abc import Iterable
 from pathlib import Path
 
 import copy
@@ -11,6 +12,7 @@ from logging import getLogger, Formatter, StreamHandler
 from typing import Any, List, Optional, Tuple
 
 import numpy as np
+import pandas as pd
 import pydicom
 
 class Defaults:
@@ -459,6 +461,18 @@ class PathData(Data):
 
         return objs
 
+    def get_file_size(self):
+        '''
+        Return size in bytes of associated file.
+        '''
+        return Path(self.path).stat().st_size if Path(self.path).exists() else 0
+
+    def get_n_file(self):
+        '''
+        Return number of data files associated with this object.
+        '''
+        # Only 1 data file associated with a non-Archive object.
+        return 1
 
 @functools.total_ordering
 class Dated(PathData):
@@ -498,6 +512,9 @@ class Dated(PathData):
         else:
             self.timestamp = f"{self.date}_{self.time}"
 
+    def get_pandas_timestamp(self):
+        return pd.Timestamp(''.join([self.date, self.time]))
+
     def in_date_interval(self,
                          min_date: Optional[str] = None,
                          max_date: Optional[str] = None) -> bool:
@@ -574,6 +591,29 @@ class Archive(Dated):
 
         self.files.sort()
 
+    def get_file_size(self):
+        '''
+        Return size in bytes of associated file.
+        '''
+        if Path(self.path).is_file():
+            size = Path(self.path).stat().st_size
+        else:
+            size = 0
+            for file in self.files:
+                if Path(file.path).exists():
+                    size += Path(file.path).stat().st_size
+
+        return size
+
+    def get_n_file(self):
+        '''
+        Return number of data files associated with this object.
+        '''
+        if Path(self.path).is_file():
+            n_file = 1
+        else:
+            n_file = len(self.files)
+        return n_file
 
 class File(Dated):
     """File with an associated date. Files can be sorted based on their
@@ -629,6 +669,7 @@ def fullpath(path: str = "") -> str:
 
     expanded = ""
     if path:
+        path = str(path)
         tmp = os.path.expandvars(path.strip())
         tmp = os.path.abspath(os.path.expanduser(tmp))
         expanded = os.path.realpath(tmp)
@@ -790,3 +831,91 @@ def get_data_by_filename(data_objects=None, remove_timestamp=True,
             data_by_filename[filename] = data_object
 
     return data_by_filename
+
+def get_n_file(objs=None):
+    '''
+    Return number of data files associated with listed objects.
+
+    **Parameter:**
+
+    objs : list, default=None
+        List of objects for which numbers of files are to be summed.
+    '''
+    if not isinstance(objs, Iterable):
+        objs = [objs]
+
+    n_file = 0
+    for obj in objs:
+        n_file += obj.get_n_file()
+    return n_file
+
+def get_file_size(objs=None):
+    '''
+    Return size in bytes of data files associated with listed objects.
+
+    **Parameter:**
+
+    objs : list, default=None
+        List of objects for which file sizes are to be summed.
+    '''
+    if not isinstance(objs, Iterable):
+        objs = [objs]
+
+    size = 0
+    for obj in objs:
+        size += obj.get_file_size()
+    return size
+
+def get_time_separated_objects(objs, min_delta=4, unit='hour',
+        most_recent=True):
+    '''
+    Return ordered list of dated objects, filtering for minimum time separation.
+
+    objs : list
+        List of dated objects.
+
+    min_delta : int/pandas.Timedelta, default=4
+        Minimum time interval required between objects.  If an integer,
+        the unit must be specified.
+
+    unit : str, default='hour'
+        Unit of min_delta if the latter is specified as an integer;
+        ignored otherwise.  Valid units are any accepted by pandas.Timedelta:
+        https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.Timedelta.html
+
+    most_recent : bool, default=True
+        When objects aren't separated by the minimum time interval, keep
+        the most recent if True, or the least recent otherwise. 
+    '''
+
+    # Ensure that min_delta is a pandas Timedelta object.
+    if not isinstance(min_delta, pd.Timedelta):
+        min_delta = pd.Timedelta(min_delta, unit)
+
+    # Only consider objects with non-null date and time.
+    timed_objs = {}
+    for obj in objs:
+        if obj.date and obj.time:
+            timestamp = pd.Timestamp(''.join([obj.date, obj.time]))
+            timed_objs[timestamp] = obj
+
+    # Obtain the list of objects with time separation greater than minimum.
+    time_separated_objs = []
+    if timed_objs:
+        # Sort time stamps,
+        # reversing order if most-recent objects are to be kept.
+        timestamps = sorted(timed_objs.keys(), reverse=most_recent)
+        time_separated_objs = [timed_objs[timestamps[0]]]
+
+        # Check that absolute time separation is greater than minimum.
+        # (Time separations will be all positive or all negative,
+        # depending on sort order.)
+        for idx in range(1, len(timestamps)):
+            if abs(timestamps[idx] - timestamps[idx - 1]) > min_delta:
+                time_separated_objs.append(timed_objs[timestamps[idx]])
+
+    # Ensure ascending order.
+    if most_recent:
+        time_separated_objs.reverse()
+
+    return time_separated_objs
