@@ -317,6 +317,7 @@ class DicomWriter:
                 'RTSTRUCT' : 'RTStructureSetStorage',
                 'CT' : 'CTImageStorage',
                 'MR' : 'MRImageStorage',
+                'PT' : 'PositronEmissionTomographyImageStorage',
                 }
 
         media_storage_sop_class_name = storage_classes.get(self.modality)
@@ -395,7 +396,13 @@ class DicomWriter:
         # Set attributes specific to 'Image' type.
         if self.source_type == 'Image':
             self.ds.ImagesInAcquisition = self.data.shape[2]
-            self.ds.RescaleSlope = getattr(self.ds, 'RescaleSlope', 1)
+            intensity_range = self.data.max() - self.data.min()
+            if intensity_range < 1000:
+                default_rescale_slope = intensity_range / 1000
+            else:
+                default_rescale_slope = 1
+            self.ds.RescaleSlope = getattr(
+                    self.ds, 'RescaleSlope', default_rescale_slope)
 
         # Set attributes specific to 'Dose' type.
         elif self.source_type == 'Dose':
@@ -528,7 +535,7 @@ class DicomWriter:
         '''
 
         for attribute, value in self.header_extras.items():
-            setattr(self.ds, attribute, value)
+            setattr(self.ds, attribute.replace(' ',''), value)
 
     def write(self):
         '''
@@ -550,7 +557,7 @@ class DicomWriter:
                 outpath = self.get_path_with_timestamp()
             else:
                 outpath = str(self.outdir / self.outname)
-            self.ds.save_as(outpath)
+            self.ds.save_as(outpath, write_like_original=False)
 
         elif self.source_type == 'Image':
             # Obtain rescale parameters.
@@ -561,7 +568,7 @@ class DicomWriter:
                 self.set_image_slice(i)
                 outname = f'{self.ds.InstanceNumber}.dcm'
                 outpath = self.outdir / outname
-                self.ds.save_as(outpath)
+                self.ds.save_as(outpath, write_like_original=False)
 
         elif self.source_type == 'StructureSet':
             self.set_structure_set()
@@ -569,6 +576,6 @@ class DicomWriter:
                 outpath = self.get_path_with_timestamp()
             else:
                 outpath = str(self.outdir / self.outname)
-            self.ds.save_as(outpath)
+            self.ds.save_as(outpath, write_like_original=False)
 
         return self.ds
