@@ -7,6 +7,8 @@ import shutil
 import time
 import timeit
 
+import pandas as pd
+
 import skrt.core
 from skrt.image import Image
 from skrt.structures import StructureSet
@@ -385,7 +387,7 @@ class Study(skrt.core.Archive):
 
         else:
 
-            # Get fraction froup and beam sequence
+            # Get fraction group and beam sequence
             if self.plans:
                 n_frac_group = self.plans[-1].nFractionGroup
                 n_beam_seq = self.plans[-1].nBeamSequence
@@ -860,10 +862,13 @@ class Patient(skrt.core.PathData):
         # Find an object from which to extract the info
         obj = None
         if self.studies:
-            obj = getattr(
-                self.studies[0],
-                f"{self.studies[0].image_types[0].lower()}_images"
-            )[-1]
+            # Preferentially choose first CT scan.
+            image_types = list(self.studies[0].image_types.keys())
+            if image_types:
+                image_type = 'ct' if 'ct' in image_types else image_types[0]
+                obj = getattr(
+                    self.studies[0],
+                    f"{image_type.lower()}_images")[0]
 
         # Read demographic info from the object
         if obj and obj.files:
@@ -879,6 +884,14 @@ class Patient(skrt.core.PathData):
         if info["Sex"]:
             info["Sex"] = info["Sex"][0].upper()
 
+        # Obtain birth date as pandas Timestamp.
+        if info["BirthDate"]:
+            info["BirthDate"] = pd.Timestamp(info["BirthDate"])
+
+        # Obtain age as an integer in years.
+        if info["Age"]:
+            info["Age"] = int(info["Age"].strip('0').strip('Y'))
+
         # Store data
         self.age = info["Age"]
         self.sex = info["Sex"]
@@ -886,19 +899,227 @@ class Patient(skrt.core.PathData):
 
     def get_age(self):
 
-        self.load_demographics()
+        if not hasattr(self, 'age'):
+            self.load_demographics()
         return self.age
 
     def get_sex(self):
 
-        self.load_demographics()
+        if not hasattr(self, 'sex'):
+            self.load_demographics()
         return self.sex
 
     def get_birth_date(self):
 
-        self.load_demographics()
+        if not hasattr(self, 'birth_date'):
+            self.load_demographics()
         return self.birth_date
 
+    def get_info(self, collections=None, data_labels=None, plan_image_type='ct',
+            treatment_image_type='mvct', min_delta=4, unit='hour', df=False):
+        '''
+        Retrieve patient summary information.
+
+        **Parameters:**
+
+        collections : dict, default=None
+            Dictionary where keys define types of patient grouping,
+            and values are lists defining specific groupings of this type.  For
+            example, {'cohort' : ['discovery', 'consolidation']} defines
+            two patient cohorts - a discovery cohort and a consolidation
+            cohort.  It's assumed that specific groupings are included
+            in data paths.  If None, an empty dictionary is used.
+
+        data_labels : list, default=None
+            List of strings specifying types of data associated with
+            a study.  If None, the list used is:
+            ['image', 'structure_set', 'dose', 'plan'].
+
+        plan_image_type : str, default='ct'
+            String identifying type of image recorded for treatment planning.
+
+        treatment_image_type : str, default='mvct'
+            String identifying type of image recorded at treatment time.
+
+        min_delta : int/pandas.Timedelta, default=4
+            Minimum time interval required between image objects.
+            If an integer, the unit must be specified.
+
+        unit : str, default='hour'
+            Unit of min_delta if the latter is specified as an integer;
+            ignored otherwise.  Valid units are any accepted by
+            pandas.Timedelta:
+            https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.Timedelta.html
+
+        df : bool, default=False
+            If False, return summary information as a dictionary.
+            If True, return summary information as a pandas dataframe.
+        '''
+        # Ensure that dictionary is defined for patient groupings.
+        if collections is None:
+            collections = {}
+
+        # Ensure that list is defined for data labels.
+        if data_labels is None:
+            data_labels = ['image', 'structure_set', 'dose', 'plan']
+
+        # Create dictionary for information to be returned.
+        info = {}
+
+        # Store basic patient information.
+        info['id'] = self.id
+        info['disease'] = self.id.split("_")[1]
+        info['birth_date'] = self.get_birth_date()
+        info['age'] = self.get_age()
+        info['sex'] = self.get_sex()
+
+        # Store information on patient groupings.
+        # The assumption here is that the collection type is
+        # included in the data path.
+        for collection, collection_types in collections.items():
+            info[collection] = None
+            for collection_type in collection_types:
+                if collection_type in self.path:
+                    info[collection] = collection_type
+
+        # Store plan information.
+        # Information is taken from the earliest plan file,
+        # in the earliest study containing a plan file.
+        info['plan_name'] = None
+        info['plan_fraction'] = None
+        info['plan_target_dose'] = None
+        for study in self.studies:
+            if hasattr(study, 'plan_types'):
+                plan_type = sorted(list(study.plan_types.keys()))[0]
+                plan = study.plan_types[plan_type][0]
+                plan.load()
+                info['plan_name'] = plan.name
+                info['plan_fraction'] = plan.n_fraction
+                info['plan_target_dose'] = plan.target_dose
+                break
+
+        # Store information about all data types, across all studies.
+        info['n_study'] = len(self.studies)
+        for study in self.studies:
+            for data_label in data_labels:
+                type_label = f'{data_label}_types'
+                data_types = getattr(study, type_label, None)
+                if data_types:
+                    # For each data type, store information on:
+                    # - number of files;
+                    # - combined size (in bytes) of all files;
+                    # For image data, also store information on:
+                    # - number of image objects.
+                    for data_type, objs in sorted(data_types.items()):
+                        file_label = f'{data_label}_{data_type}_file'
+                        size_label = f'{data_label}_{data_type}_size'
+                        obj_label = f'{data_label}_{data_type}_obj'
+                        if not file_label in info:
+                            info[file_label] = 0
+                        if not size_label in info:
+                            info[size_label] = 0
+                        if 'image' == data_label:
+                            if obj_label not in info:
+                                info[obj_label] = 0
+                            info[obj_label] += len(objs)
+                        for obj in objs:
+                            info[file_label] += obj.get_n_file()
+                            info[size_label] += obj.get_file_size()
+
+        # Create lists of image objects for planning and for treatment.
+        plan_images = self.combined_objs(f'{plan_image_type}_images')
+        treatment_images = self.combined_objs(f'{treatment_image_type}_images')
+
+        # Store time of image used for plan creation.
+        if plan_images:
+            info['plan_image_time'] = plan_images[0].get_pandas_timestamp()
+        else:
+            info['plan_image_time'] = None
+
+        # Store time of plan creation.
+        if info['plan_fraction'] is not None:
+            info['plan_time'] = plan.get_pandas_timestamp()
+        else:
+            info['plan_time'] = None
+
+        # Store times of first and last treatment images.
+        if treatment_images:
+            info['treatment_start'] = treatment_images[0].get_pandas_timestamp()
+            info['treatment_end'] = treatment_images[-1].get_pandas_timestamp()
+        else:
+            info['treatment_start'] = None
+            info['treatment_end'] = None
+
+        # Filter to have images separated by a minimum amount of time.
+        time_separated_plan_images = skrt.core.get_time_separated_objects(
+                plan_images, min_delta=min_delta, unit=unit)
+        time_separated_treatment_images = skrt.core.get_time_separated_objects(
+                treatment_images, min_delta=min_delta, unit=unit)
+
+        # Store number of time-separated images.
+        info['n_treatment'] = len(time_separated_treatment_images)
+
+        return (pd.DataFrame([info]) if df else info)
+
+    def get_image_info(self, image_types=None, min_delta=4, unit='hour',
+            df=False):
+        '''
+        Retrieve information about images.
+
+        **Parameters:**
+
+        image_types : list, default=None
+            List of strings indicating types of image for which information
+            is to be retrieved, for example ['ct', 'mr'].  If None,
+            information is retrieved for all image types in the patient
+            dataset.
+
+        min_delta : int/pandas.Timedelta, default=4
+            Minimum time interval required between image objects.
+            If an integer, the unit must be specified.
+
+        unit : str, default='hour'
+            Unit of min_delta if the latter is specified as an integer;
+            ignored otherwise.  Valid units are any accepted by
+            pandas.Timedelta:
+            https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.Timedelta.html
+
+        df : bool, default=False
+            If False, return summary information as a dictionary.
+            If True, return summary information as a pandas dataframe.
+        '''
+        if image_types is None:
+            all_image_types = self.combined_objs('image_types')
+            image_types = []
+            for image_types_now in all_image_types:
+                for image_type in image_types_now:
+                    if image_type not in image_types:
+                        image_types.append(image_type)
+
+        all_info = []
+        for image_type in sorted(image_types):
+            image_label = f'{image_type.lower()}_images'
+            images = self.combined_objs(image_label)
+            time_separated_images = skrt.core.get_time_separated_objects(
+                    images, min_delta=min_delta, unit=unit)
+            ref_time = None
+            for image in time_separated_images:
+                info = {}
+                info['id'] = self.id
+                info['modality'] = image_type
+                info['timestamp'] = image.get_pandas_timestamp()
+                info['day'] = info['timestamp'].isoweekday()
+                if ref_time is not None:
+                    info['time_delta'] = info['timestamp'] - ref_time
+                    info['days_delta'] = info['time_delta'].round('1d')
+                else:
+                    info['time_delta'] = None
+                    info['days_delta'] = None
+                ref_time = info['timestamp']
+                all_info.append(info)
+
+        return (pd.DataFrame(all_info) if df else all_info)
+
     def get_subdir_studies(self, subdir=""):
         """Get list of studies within a given subdirectory."""
 
